<?php

namespace Util;

use Teknisa\Libs\Util\SMTP;
use Teknisa\Libs\Util\Utilities;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;

class SupUtilities
{
    protected $connection;
    protected $newCode;
    protected $newCodeSC;
    protected $fileServerURL;
    protected $crudService;
    protected $libUtilities;
    protected $dataSourceManager;

    private $isUsingCustomSmtp = false;
    private $customSmtpSettings = array();

    const TABLE_LISTA = "LISTA";

    public function __construct($connection, $newCode, $newCodeSC, $environment, $fileServerURL, $libUtilities, $crudService, $dataSourceManager)
    {
        $this->connection        = $connection;
        $this->newCode           = $newCode;
        $this->newCodeSC         = $newCodeSC;
        $this->environment       = $environment;
        $this->fileServerURL     = $fileServerURL;
        $this->crudService       = $crudService;
        $this->libUtilities      = $libUtilities;
        $this->dataSourceManager = $dataSourceManager;
    }

    public function iniciaTransacao()
    {
        $this->connection->beginTransaction();
    }

    public function commit()
    {
        $this->connection->commit();
    }

    public function flush()
    {
        $this->connection->flush();
    }

    public function rollback()
    {
        $this->connection->rollback();
    }

    /**
     * Receive a query and param and check if operation is a "SELECT",
     * if is, return the result. The param passed along with query are 
     * not mandatory.
     * 
     * @param $query
     * @param $params
     * @return array
     * @throws \Exception
     */
    public function crud($query, $params = [])
    {
        try {
            $std = $this->connection->prepare($query);
            $std->execute($params);
            if (self::getQueryOperation($query)) {
                return $std->fetchAll();
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Returns true if query operation is a "SELECT" and false if
     * is another one.
     * 
     * @param string $query
     */
    public function getQueryOperation($query)
    {
        return strpos(strtoupper(ltrim($query)), 'SELECT') === 0 ? true : false;
        // $operation = substr($query, 0, 6);
        // return strtoupper($operation) === 'SELECT' ? true : false;
    }

    /**
     * Execute a newCode on a table name received.
     * 
     * @param string $tableName
     */
    public function generateCode($tableName)
    {
        $newCode = $this->newCode->getCodeByTableName($tableName, $this->environment->getNrOrgTrab());
        return $newCode;
    }

    /**
     * Execute a newCodeSC on a table name received.
     * 
     * @param string $tableName
     */
    public function generateCodeSC($tableName)
    {
        $newCode = $this->newCodeSC->getCodeByTableName($tableName, $this->environment->getNrOrgTrab());
        return $newCode;
    }

    /**
     * Define Cookie 
     */
    public function defineCookieEmail($name, $value = "", $time = 0, $path = "/", $domain = "", $secure = false, $httponly = false)
    {

        setcookie($name, $value, $time, $path, $domain, $secure, $httponly);
    }

    /**
     * Retorna um Cookie pelo nome
     */
    public function retornaCookieEmail($name)
    {

        if (array_key_exists($name, $_COOKIE)) {
            return $_COOKIE[$name];
        }

        return false;
    }

    public function retornarEmailsRelacionadosFornecedor(array $fornecedores)
    {
        $listaEmails = array();

        $fornecedores = array_unique($fornecedores);

        foreach ($fornecedores as $fornecedor) {
            $data = $this->emailsRelacionadosFornecedor($fornecedor)->getRows();
            if(isset($data) && count($data)) {
                $emailFornecedores = end($data);
                if ($emailFornecedores['DSLOGINECO']) {
                    $listaEmails = array_merge($listaEmails, explode(',', $emailFornecedores['DSLOGINECO']));
    
                    if ($emailFornecedores['DSEMAILECO']) {
                        $listaEmails = array_merge($listaEmails, explode(',', $emailFornecedores['DSEMAILECO']));
                    }
                }
            }            
        }
        
        if (!count($listaEmails)) {
            throw new \Exception("SUPPLIER_EMAIL_NOT_REGISTERED");
        }

        $listaEmails = array_unique($listaEmails);

        return $listaEmails;
    }

    public function emailsRelacionadosFornecedor($cdforneced)
    {
        $conditions = [
            array(
                "columnName" => "CDFORNECED",
                "operator"   => '=',
                "value"      => $cdforneced
            )
        ];

        return $this->crudService->getFilteredDataSet('emailsRelacionadosFornecedor', $conditions);
    }

    public function fileGetContentUtf8($fileName)
    {
        $content = file_get_contents($fileName);
        return mb_convert_encoding($content, 'UTF-8', mb_detect_encoding($content, 'UTF-8, ISO-8859-1', true));
    }

    public function returnLogo($idAplicativo)
    {
        $sql = "SELECT URLLOGO FROM APPCONFIG WHERE IDAPLICATIVO = :IDAPLICATIVO";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute(
            array(
                "IDAPLICATIVO" => $idAplicativo,
            )
        );
        $result = $stmt->fetchAll();
        if (isset($result[0]) && $result[0]['URLLOGO'] && $result[0]['URLLOGO'] != null && $result[0]['URLLOGO'] != '') {
            return $this->fileServerURL . $result[0]['URLLOGO'];
        } else {
            return false;
        }
    }

    public function returnLgpd($cdFilial)
    {
        $sql = "SELECT P.TXAVISOLEGALEMAIL FROM PARASUPRI P, FILIAL F WHERE F.CDFILIAL = :P_CDFILIAL AND P.CDFILIAL = NVL(F.CDFILCENTCMP, F.CDFILIAL)";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute(
            array(
                "P_CDFILIAL" => $cdFilial
            )
        );
        $result = $stmt->fetchAll();
        return $result[0]['TXAVISOLEGALEMAIL'];
    }

    public function messageSendMail($cdFilial, $assunto, $message, $idAplicativo = 'PORTAL E-COMMERCE', $attrImg = 'width="175px"')
    {
        $logoSrc = $this->returnLogo($idAplicativo);
        $this->getLogoAttributes($logoSrc, $attrImg);
        $lgpd = $this->returnLgpd($cdFilial);
        $messageSendMail = $this->returnMessageSendMail($logoSrc, $assunto, $message, $attrImg, $lgpd);
        return $messageSendMail;
    }

    public function checkFileExists($url)
    {
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_NOBODY, true);
        curl_exec($ch);
        $code = curl_getinfo($ch, CURLINFO_HTTP_CODE);

        if ($code == 200) {
            $status = true;
        } else {
            $status = false;
        }

        curl_close($ch);
        return $status;
    }

    public function getLogoAttributes(&$logoSrc, &$attrImg)
    {
        if ($logoSrc != false && $this->checkFileExists($logoSrc)) {
            $imgAttributes = getimagesize($logoSrc);

            if ($imgAttributes[1] > $imgAttributes[0]) {
                $attrImg = 'height="175px"';
            }
        }
    }

    public function returnMessageSendMail($logoSrc, $assunto, $message, $attrImg, $lgpd)
    {
        $templatesDir = realpath(__DIR__ . '/Template');
        if ($logoSrc != false && $this->checkFileExists($logoSrc)) {
            $templateEmail = $this->fileGetContentUtf8($templatesDir . "/email.html");
            $replace = array(
                $attrImg,
                $logoSrc,
                $assunto,
                $message,
                $lgpd
            );
    
            $varReplaced = array(
                '%attrImg%',
                '%logoSrc%',
                '%assunto%',
                '%mensagem%',
                '%lgpd%'
            );
        } else {
            $templateEmail = $this->fileGetContentUtf8($templatesDir . "/email_without_logo.html");
            $replace = array(
                $attrImg,
                $assunto,
                $message,
                $lgpd
            );
    
            $varReplaced = array(
                '%attrImg%',
                '%assunto%',
                '%mensagem%',
                '%lgpd%'
            );
        }

        return $messageSendMail = str_replace($varReplaced, $replace, $templateEmail);
    }

    public function enviaEmail($fromName, $subject, $address, $messageSendMail, $emailDestino, $documentoAnexoFullPath = false, $name = false, $type = false, $filial = null)
    {
        try {
            $this->useCustomSmtp($filial);
            if (!$this->isUsingCustomSmtp && $fromName != '') {
                $this->smtpUtil->setFromName($fromName);
                $this->smtpUtil->setNameReplyTo($fromName);
            }

            if ($documentoAnexoFullPath) {
                $this->smtpUtil->clearAttachments();
                $this->smtpUtil->addAttachment(file_get_contents($documentoAnexoFullPath), $name, 'base64', $type, 'attachment', true);
            }

            if ($address != '') {
                $this->smtpUtil->setNmAddress($address);
            }

            $this->smtpUtil->setSubject($subject);
            $this->smtpUtil->sendMail($messageSendMail, $emailDestino);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function useCustomSmtp($filial = null)
    {
        if (!$this->isUsingCustomSmtp) {
            $this->customSmtpSettings = $this->getCustomSmtpSettings($filial);
        }

        $this->smtpUtil = new SMTP($this->customSmtpSettings);

        if (isset($this->customSmtpSettings['fromName'])) {
            $this->smtpUtil->setFromName($this->customSmtpSettings['fromName']);
        }

        if (isset($this->customSmtpSettings['from'])) {
            $this->smtpUtil->setNameReplyTo($this->customSmtpSettings['from']);
        }
        
        if (isset($this->customSmtpSettings['username']) && isset($this->customSmtpSettings['password'])) {
            if (!empty($this->customSmtpSettings['username']) && !empty($this->customSmtpSettings['password'])) {
                $this->smtpUtil->setSMTPAuth(true);
            }
        }

        $this->isUsingCustomSmtp = true;
    }

    public function getCustomSmtpSettings($filial = null)
    {
        try{
            $tipoDadoUtilizado = $this->getTipoDadoUilizadoEnvioEmail($filial)['IDUTDADOPERENVEMAIL'];
            $dadosSMTPFilial   = $this->getDadosSMTPFromFilial($filial);
            $settings          = self::getSmtpSettingsFromAddress($dadosSMTPFilial['DSSMTPEMAILF']);
            if (isset($settings['host']) && !empty($settings['host'])) {
                $params['host'] = $settings['host'];
                if (isset($settings['port'])) {
                    $params['port'] = $settings['port'];
                }
                
                switch ($tipoDadoUtilizado) {
                    case 'LOGIN_FILIAL_REMETENTE_FILIAL':
                        $params['from']     = $dadosSMTPFilial['DSLOGINSMTP'];
                        $params['fromName'] = $dadosSMTPFilial['DSLOGINSMTP'];
                        $params['username'] = $dadosSMTPFilial['DSLOGINSMTP'];
                        $params['password'] = $dadosSMTPFilial['CDSENHEMAILF'];
                        break;
                    case 'LOGIN_FILIAL_REMETENTE_OPER':
                        $dadosEmailOperadorLogado = $this->getDadosEmailOperadorLogado();
        
                        $params['from']     = $dadosEmailOperadorLogado['DSEMAILOPER'];
                        $params['fromName'] = $dadosEmailOperadorLogado['NMOPERADOR'];
                        $params['username'] = $dadosSMTPFilial['DSLOGINSMTP'];
                        $params['password'] = $dadosSMTPFilial['CDSENHEMAILF'];
                        break;
                    case 'LOGIN_OPER_REMETENTE_OPER':
                        $dadosEmailOperadorLogado = $this->getDadosEmailOperadorLogado();
        
                        $params['from']     = $dadosEmailOperadorLogado['DSEMAILOPER'];
                        $params['fromName'] = $dadosEmailOperadorLogado['NMOPERADOR'];
                        $params['username'] = $dadosEmailOperadorLogado['DSEMAILOPER'];
                        $params['password'] = $dadosEmailOperadorLogado['CDSENEMAILOP'];
                        break;
                }
        
        
                $params['password'] = $this->getPassword($params, $tipoDadoUtilizado);
        
                $securityParameters = $this->getSecurityParameters($dadosSMTPFilial, $params);
                $params = array_merge($params, $securityParameters);
        
                return $params;
            } else {
                return array();
            }
        } catch(\Exception $e) {
            return array();
        }
    }

    /**
     * Recupera a senha descriptografada. Caso a senha não exista nos parâmetros,
     * retorna uma string vazia.
     * 
     * @param array $customSmtpSettings
     * 
     * @return string Contendo a senha devidamente tratada.
     */
    private function getPassword($customSmtpSettings, $tipoDadoUtilizado)
    {
        if (!isset($customSmtpSettings['password']) || empty($customSmtpSettings['password'])) {
            return '';
        }

        if ($tipoDadoUtilizado == 'LOGIN_OPER_REMETENTE_OPER') {
            return $this->libUtilities->getDelphiPasswordEncrypted($customSmtpSettings['password'], false);
        }

        return self::getDelphiPasswordEncrypted($customSmtpSettings['password'], false);
    }

    /**
     * Recupera informações de segurança para usar na conexão com o SMTP.
     * Utiliza dos parâmetros recuperados para definir as seguintes propriedades:
     * . smtpSecure:
     *      - 'tls', caso o parâmetro IDUTTLSEMAIL esteja marcado;
     *      - 'ssl', caso o parâmetro IDUTSSLEMAIL esteja marcado;
     *      - '', caso ambos os parâmetros acima estejam desmarcados;
     * . smtpAuth:
     *      - true, caso o usuário e senha estejam preenchidos;
     *      - false, caso usuário ou senha estejam em branco;
     * 
     * @param array $dadosSMTPFilial
     * @param array $customSmtpSettings
     * 
     * @return array Contendo os parâmetros de segurança.
     */
    private function getSecurityParameters($dadosSMTPFilial, $customSmtpSettings)
    {
        $params = [
            'smtpSecure' => '',
            'smtpAuth' => false
        ];

        if ($dadosSMTPFilial['IDUTTLSEMAIL'] == 'S') {
            $params['smtpSecure'] = 'tls';
        } else if ($dadosSMTPFilial['IDUTSSLEMAIL'] == 'S') {
            $params['smtpSecure'] = 'ssl';
        }

        if (!empty($customSmtpSettings['username']) && !empty($customSmtpSettings['password'])) {
            $idsisdos = $this->getIdSisDos();
            if ($idsisdos != 3) {
                $params['smtpAuth'] = true;
            }
        }
        return $params;
    }

    /**
     * Recupera informações do endereço de SMTP, como:
     * - host;
     * - porta;
     * 
     * @param string $url Url do servidor com ou sem porta.
     * 
     * @return array Contendo as informações encontradas.
     */
    public function getSmtpSettingsFromAddress($url)
    {
        $match = self::splitByCommonDelimiters($url);

        if (empty($match)) {
            throw new \Exception('Formato do endereço de SMTP inválido.');
        }

        if (count($match) > 1) {
            $params['port'] = $match[1];
        }

        $params['host'] = $match[0];

        return $params;
    }

    public function splitByCommonDelimiters($string)
    {
        $pattern = '/[:;|]+/';
        $cleanedString = self::removeAllWhitespaces($string);

        $splittedString = preg_split($pattern, $cleanedString);

        return $splittedString;
    }

    public function removeAllWhitespaces($string)
    {
        $pattern = '/\s+/';

        $cleanedString = preg_replace($pattern, '', $string);

        return $cleanedString;
    }

    public function getTipoDadoUilizadoEnvioEmail($filial = null)
    {
        $query = "SELECT PARASUPRI.IDUTDADOPERENVEMAIL
                        FROM PARASUPRI
                    WHERE PARASUPRI.CDFILIAL = :P_CDFILCENTCMP";
        $std = $this->connection->prepare($query);

        $std->execute(array(
            "P_CDFILCENTCMP" => self::getCentralCompras($filial ? $filial : $this->environment->getCdFilial())['CDFILCENTCMP'],
        ));

        return $std->fetch();
    }

    public function getCentralCompras($codigoFilial = null)
    {
        if (!$codigoFilial) {
            $codigoFilial = $this->environment->getCdFilial();
        }

        $query = "SELECT F.CDFILIAL, F.NMFILIAL, F2.CDFILIAL AS CDFILCENTCMP,
                         F2.NMFILIAL AS NMFILCENTCMP, F2.NRORG
                    FROM FILIAL F, FILIAL F2
                   WHERE F.CDFILIAL = :CDFILIAL
                     AND F2.CDFILIAL = NVL(F.CDFILCENTCMP, F.CDFILIAL)";
        $std = $this->connection->prepare($query);

        $std->execute(array(
            "CDFILIAL" => $codigoFilial
        ));

        return $std->fetch();
    }

    public function getDadosSMTPFromFilial($filial = null)
    {
        $query = "SELECT EF.CDSENHEMAILF, EF.DSSMTPEMAILF, NVL(EF.DSLOGINSMTP, EF.DSEMAILFILI) DSLOGINSMTP,
                         NVL(EF.DSEMAILFILI, EF.DSLOGINSMTP) DSEMAILFILI, EF.IDUTTLSEMAIL,
                         EF.IDUTSSLEMAIL, FI.CDFILIAL||' - '||FI.NMFILIAL AS NMFILIAL
                    FROM ENDEFILI EF INNER JOIN FILIAL FI ON EF.CDFILIAL = NVL(FI.CDFILCENTCMP, FI.CDFILIAL)
                   WHERE FI.CDFILIAL     = :P_CDFILIPROC
                     AND EF.IDTPENDEFILI = 'P'";
        $std = $this->connection->prepare($query);

        $std->execute(array(
            "P_CDFILIPROC" => $filial ? $filial : $this->environment->getCdFilial(),
        ));

        return $std->fetch();
    }

    public function getDadosEmailOperadorLogado()
    {
        $query = "SELECT OP.NMOPERADOR, OP.DSEMAILOPER, NVL(OP.CDSENEMAILOP, OP.CDSENHOPER) CDSENEMAILOP
                    FROM OPERADOR OP
                   WHERE OP.CDOPERADOR = :P_CDOPERADOR";
        $std = $this->connection->prepare($query);

        $std->execute(array(
            "P_CDOPERADOR" => $this->environment->getCdOperador(),
        ));

        return $std->fetch();
    }

    public function getVisaoSistema()
    {
        $filialLogada = $this->environment->getCdFilial();
        $centralCompras = $this->getCentralCompras($filialLogada)['CDFILCENTCMP'];

        if ($centralCompras == $filialLogada) {
            return 'SUP';
        }

        return 'CMP';
    }

    /**
     * Temporário até que uma solução seja implementada no local original desta
     * função.
     * 
     * A original realiza chamada apenas nas funções GetCriptoPass e GetUnCriptoPass,
     * equivalentes ao CriptoPlus e UncriptoPlus do Delphi.
     * 
     * Porém, utilizamos as criptografias convencionais, que até então não eram
     * implementadas la Bib_html.
     * 
     * Foi incluída na 'Bib', rdm 'rdmDBGeral', na issue #238324
     * 
     * Teknisa\Libs\Util\Utilities
     */
    public static function getDelphiPasswordEncrypted($password, $isEncrypt = true, $offset = '2')
    {
        $url = self::getAspUrlParameter() . '/comwebservice/service.asmx?wsdl';

        $options = array(
            'encoding'    => 'ISO-8859-1',
            'trace'       => true,
            'exceptions'  => true,
            'compression' => SOAP_COMPRESSION_ACCEPT | SOAP_COMPRESSION_GZIP,
        );

        $args = array(
            new \SoapVar($password, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
            new \SoapVar($offset, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
        );

        $cryptType = $isEncrypt ? 'GetCripto' : 'GetUncripto';

        $params = array(
            'rdm'    => 'Bib.rdmDBGeral',
            'method' => $cryptType,
            'args'   => $args,
        );

        try {
            $client = new \SoapClient($url, $options);
            $result = $client->executeCOMMethod($params);
            $encryption = $result->executeCOMMethodResult;
        } catch (\SoapFault $e) {
            throw new \Exception($e->getMessage());
        }

        return $encryption;
    }

    /**
     * Temporário até que uma solução seja implementada para a função getDelphiPasswordEncrypted.
     * 
     * Teknisa\Libs\Util\Utilities
     */
    public static function getAspUrlParameter()
    {
        return InstanceManager::getInstance()->getParameter('aspUrl');
    }

    /**
     * Transforma os parâmetros de um request do tipo Filter (array de array)
     * em row (array chave/valor), facilitando o acesso ao valor através do nome.
     * Ex.:
     * - Antes:  $array[0]['value'] para recuperar o valor do primeiro parâmetro.
     * - Depois: $array['columnName'] para recuperar o valor do parâmetro 'columnName'.
     * 
     * @param Zeedhi\Framework\DTO\Request\Filter $request
     * 
     * @return array Contendo o nome e valor dos parâmetros encontrados.
     */
    public function getParametersFromCondititions(\Zeedhi\Framework\DTO\Request\Filter $request)
    {
        $conditions = $request->getFilterCriteria()->getConditions();

        if (empty($conditions)) return $conditions;

        return array_merge(...array_map(function ($param) {
            return [$param['columnName'] => $param['value']];
        }, $conditions));
    }

    public function getIdSisDos()
    {
        $query = "SELECT IDSISDOS FROM PARAMGERAL";

        $std = $this->connection->prepare($query);
        $std->execute();
        $idSisDos = $std->fetch();

        return empty($idSisDos) ? false : $idSisDos['IDSISDOS'];
    }

    public function insertLista($cdlistaArray)
    {
        $dataSourceName = strtolower(self::TABLE_LISTA);
        $nrlista = $this->newCode->getCodeByTableName(self::TABLE_LISTA, $this->environment->getNrOrg());
        $rows = array();
        foreach ($cdlistaArray as $cdlista) {
            $row = array(
                "NRLISTA" => $nrlista,
                "CDLISTA" => $cdlista,
                "__is_new" => true
            );
            array_push($rows, $row);
        }
        $dataSet = new DataSet($dataSourceName, $rows);
        $request = new DTO\Request\DataSet($dataSet, "", "", "");
        $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
        $this->crudService->save($dataSourceName, $rowsToSave);
        return $nrlista;
    }

    public function insertList($nrlista, $cdlista)
    {
        $params = array(
            "NRLISTA" => $nrlista,
            "CDLISTA" => $cdlista
        );

        $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA) VALUES (:NRLISTA, :CDLISTA)";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }

    public function criarListaValores($lista)
    {
        $nrlista = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrOrg());
        $valores = explode('_', $lista);
        try {
            foreach ($valores as $key) {
                $this->insertList($nrlista, $key);
            }
            return $nrlista;
        } catch (\Exception $e) {
            throw $e;
        }
    }


    public function deleteLista($nrlista)
    {
        $dataSourceName = strtolower(self::TABLE_LISTA);
        $conditions = array(
            array(
                "columnName" => "NRLISTA",
                "operator"   => '=',
                "value"      => $nrlista
            )
        );
        $dataSet = $this->crudService->getFilteredDataSet($dataSourceName, $conditions);
        $this->dataSourceManager->delete($dataSet);
    }

    public function getMensagemPadraoEmail($operacao)
    {
        try {
            $conditions = array(
                array(
                    'columnName' => 'IDOPERACAO',
                    'operator' => '=',
                    "value" => $operacao
                )
            );

            return $this->crudService->getFilteredDataSet("mensagemPadraoEmail", $conditions)->getRows();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getFilteredDataSet($dataSourceName, $conditions) 
    {
		return $this->crudService->getFilteredDataSet($dataSourceName, $conditions);
    }
}
